{% load i18n %}

<h3>{% trans 'Description:' %}</h3>
<p>{% blocktrans %}<strong>Name</strong> is a human-readable name of a package.{% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Categories</strong> are a predefined set of values used to filter the packages.{% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Tags</strong> are an arbitrary comma-separated values also used to filter the packages.{% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Public</strong> defines whether or not should be package shown to the other tenants.{% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Active</strong> is used for temporarily disabling a package.{% endblocktrans %}</p>
<p>{% blocktrans %}<strong>Description</strong> consists of several sentences about the package's purpose.{% endblocktrans %}</p>
