{% load i18n %}
{% load url from future %}
{% load static %}
{% block page_header %}
  <div class='page-header'>
      <h2>
          {% blocktrans %}Environment: {{ environment_name }}{% endblocktrans %}
      </h2>

      <h3>
          <a href=" {% url 'horizon:murano:environments:index' %}">
              {% blocktrans %}environments{% endblocktrans %}
          </a> > environment {{ environment_name }}
      </h3>
  </div>
{% endblock %}
