{% load static %}
{% load humanize %}

<div class="well">
    <div class="app-icon pull-left server">
      <img  src="{% static 'muranodashboard/images/murano_srv.png' %}"/>
    </div>

    <h4>API server: {{ server.host }}</h4>
    <div class="server-stats">
       CPU Load:
       <meter value="{{ server.cpu_percent }}" max="{{ server.max_cpu}}">{{ server.cpu_percent }}%</meter>
       Requests:  <span class="label label-default">{{ server.request_count }}</span>
       Errors:  <span class="label label-default">{{ server.error_count }}</span>
       Ave.Resp.Time: <span class="label label-default">{{ server.average_response_time|floatformat:3}}</span>
       Req\sec: <span class="label label-default">{{ server.requests_per_second|floatformat:3 }}</span>
       Err\sec: <span class="label label-default">{{ server.errors_per_second|floatformat:3 }}</span>
    </div>

</div>