<tr>
    <td>
     {{ instance.service }}
    </td>
    <td>
        {{ instance.service_type }}
    </td>
    <td>
        {{ instance.duration }} sec.
    </td>
    <td>
        {{ instance.active }}
    </td>

</tr>
