{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Murano Statistics" %}{% endblock %}

{% block page_header %}
  {% include "stats/_page_header.html" with title=_("Murano Server Stats") %}
{% endblock page_header %}

{% block main %}
<div class="row">
  <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
  {{ tab_group.render }}
  </div>
</div>
{% endblock %}
