{% load i18n %}
{% load url from future %}
{% block page_header %}
  <div class='page-header'>

      <h2> {% blocktrans %}Component Details: {{ service_name }} {% endblocktrans %}
      </h2>
      <h3>
          <a href=" {% url 'horizon:murano:environments:index' %}">
              {% blocktrans %}environments{% endblocktrans %}
          </a> >
          <a href=" {% url 'horizon:murano:environments:services' service.environment_id %}">
              {% blocktrans %}environment {{ environment_name }}{% endblocktrans %}
          </a>
          {% blocktrans %} > component {{ service_name }}{% endblocktrans %}

      </h3>
  </div>
{% endblock %}
