.table_header.catalog {
  border-bottom: 2px solid #e5e5e5;
  padding-bottom:5px;
  margin-bottom: 10px;
}
.table_search {
  min-width: 265px;
}

.catalog .table_actions {
  width: 100%;
}

h3.heading_switcher {
  font-size: 18px;
}

h3.heading_switcher a.btn {
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.catalog .table_actions a, .catalog .table_actions button {
  margin-left: 0;
}

.catalog .row_actions button, .catalog .row_actions a {
  background: none;
  float: none;
  display: block;
  padding: 5px 10px;
  color: black;
  text-align: left;
  border-radius: 0;
  border: 0 none;
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
}

.catalog .dropdown-menu li > a:hover, .catalog .dropdown-menu .active > a, .catalog .dropdown-menu .active > a:hover {
  color: black;
  background-color: #CDCDCD;
}

.catalog .dropdown.open .dropdown-toggle {
  color: black;
  background: #e6e6e6;
}

.app-list .actions {
  border-top:1px #ccc solid;
  padding-top:15px;
  margin-top:15px;
}
.app-list .app-description {
  height: 135px;
}
.btn-link {
  padding: 5px 0;
}
.centering {
  width: 100%;
  text-align: center;
}
h3.link {
  color: yellow;
  opacity: 0.5;
}
h3.link:hover {
  opacity: 1;
}
h3.link .caret {
  border-top: 4px solid yellow !important;
}
.well:hover {
    float: none;
    border: 1px solid #bbb;
    background: #eee;
    -o-transition: all .15s;
    -moz-transition: all .15s ease-out;
    -webkit-transition: all .15s ease-out;
    transition: all .15s ease-out;
}
.row.app-list {
  margin-top: 20px;
  margin-bottom:20px;
}
.app-list .app {
  position:relative;
  -moz-border-radius: 8px;
  -webkit-border-radius: 8px;
  border-radius: 8px;
}
.app-list .app > div {
  margin:0;
}
.app-list .app div.description {
  margin-left:0;
  height: 100%;
}
.app-list .app h4 {;
  margin-bottom:0.25em;
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
}
.app-list .app p {
  max-height: 75px;
  overflow: hidden;
}

.app-list .app .app-actions {
  float: right;
  margin-top: 5px;
}
.app-actions div[class*='span'] {
  margin-left:0;
}
.header.dropdown {
  display:inline-block;
  margin-left: 5px;
}
.header.dropdown ul {
  font-size:14px;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}
.nav-tabs {
  margin-top:20px;
}
.app-meta {
  padding:10px;
  background-color: #eeeeee;
  -moz-border-radius: 8px;
  -webkit-border-radius: 8px;
  border-radius: 8px;
}
.tab-content {
  margin-bottom:10px;
}
.app-meta .app-supplier {
  border-top:1px solid #ddd;
  padding-top:15px;
  margin-top:15px;
}
.app-supplier-data {
  overflow: hidden;
  padding-left: 10px;
}

.app-meta ul {
    padding: 15px 0;
}
.app-icon {
  line-height:0;
  overflow:hidden;
  margin-left:0;
  padding: 0;
  float: left;
  max-width: 80px;
}
.app-icon img {
    max-height:70px;
    margin: 0;
}

#environment_switcher {
  display: inline-block;
}
.panel-visible{
  display: block;
}

.columns{
  clear: both;
  width: 330px;
  padding: 0 0 20px 0;
  line-height: 22px;
}

.colleft{
  float: left;
  width: 150px;
  line-height: 22px;
}

.colright{
  float: right;
  width: 150px;
  line-height: 22px;
}
.server-stats {
  margin: 10px 0;
}
.server-stats .label {
  margin-right:12px;
}
.server-stats meter {
  margin-right:20px;
}
.app-icon.server {
  margin-right:20px;
}

.no-results {
  margin-bottom: 25px;
}
.app_requirements ul li {
  font-weight: bold;
}
.app_requirements ul li ul {
  margin-left: 30px;
}
.app_requirements ul li ul li {
  font-weight: normal;
}

/* Bootstrap 3 grid always applies 15px left and right padding on grid columns */
/* special class to override first and last column in row */
.col-row:first-child {
  padding-left:0;
}
.col-row:last-child {
  padding-right: 0;
}

/* LANDSCAPE PHONE TO SMALL DESKTOP & PORTRAIT TABLET */
/* Workarounds/hacks for filter forms at top of catalog page when fluid grid... */
/* changes layout due to device/width settings in Bootstrap 3's grid  */
@media (max-width: 767px) {
  .col-xs-12 .pull-right {
    float: none !important;
  }

  .col-xs-12.col-row {
    padding-left: 0;
  }

  .col-xs-12 .table_actions form {
    float: none;
    margin-left: 0;
  }

  .col-xs-12 .table_search {
    margin-left: -5px;
  }

  .col-xs-12 .table_search .form-group {
    float: left;
    margin-right: 15px;
  }

  .col-xs-12 .table_search .form-control-feedback {
    top: 0;
  }

  .app-list .app .app-details-link {
    float: left;
  }

  .app-list .app .app-actions {
    float: left;
  }
}
