{% extends 'common/_modal_form.html' %}
{% load i18n %}
{% load url from future %}

{% block form_id %}{% endblock %}
{% block form_action %}{% url 'horizon:murano:packages:modify' app_id %}{% endblock %}

{% block modal-header %}{% trans 'Modify Package' %}{% endblock %}
{% block modal_id %}modify_package_modal{% endblock %}
{% block modal-body %}
<div class='left'>
    <fieldset>
    {% include 'horizon/common/_form_fields.html' %}
    </fieldset>
</div>
<div class='right'>
    {% include 'packages/_package_params.html' %}
</div>
{% endblock %}

{% block modal-footer %}
    {{ form.media }}
    <input class='btn btn-primary pull-right' type='submit' value='{% trans 'Submit' %}' />
    <a href="{% url 'horizon:murano:packages:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
