{% load i18n %}
<p>{% blocktrans %} Environment defines the run time context for the application including
    all the application images, what container services they run on, and the associated
    automatically generated network configuration.{% endblocktrans %}</p>
