{% load i18n %}
{% load url from future %}
{% load static %}

{% block page_header %}
<link rel="stylesheet" href="{% static 'muranodashboard/css/catalog.css' %}">
  <div class='page-header'>

      <h2> Murano Status
      </h2>

  </div>
{% endblock %}