{% load i18n sizeformat %}

{% if stats %}
    <div class="panel-group {{ offset }}" id="{{ grp_id }}">
        {% for env in stats %}
          {% with parent=grp_id env=env t="stats/_billing_stats.html" %}
            {% include t %}
          {% endwith %}
        {% endfor %}
    </div>
{% else %}
    <div class="alert alert-block alert-info fade in">
        <p>{% trans "No recent activity to report at this time." %}</p>
    </div>
{% endif %}
