<div class="panel panel-default">
  <div class="panel-heading">
    <h4 class="panel-title">
      <a data-toggle="collapse" data-parent="#{{ parent }}" href="#{{ env.id }}">
        {{ env.name }}
      </a>
    </h4>
  </div>
  <div id="{{ env.id }}" class="panel-collapse collapse in">
    <div class="panel-body">
       <table class="table">
           <thead>
            <tr class="tablesorter-headerRow">
             <th> Component Name </th>
             <th> Component Type </th>
             <th> Duration </th>
             <th> Active </th>
            </tr>
           </thead>
       {% for entry in env.instances %}
         {% with instance=entry t="stats/_billing_entry.html" %}
           {% include t %}
         {% endwith %}
        {% endfor %}
       </table>
    </div>
  </div>
</div>
